@echo off & SETLOCAL & SETLOCAL ENABLEDELAYEDEXPANSION & CLS & echo.

REM :: "BuildTrustList" Version 2.5 by Veit Berwig in 06/2024
REM :: 
REM :: --- encoded in utf-8 ---
REM :: 
REM :: -------------------- config block ----------------------- BEGIN
REM :: SET SCRIPTNAME
SET SCRIPTNAME=BuildTrustList
echo.%SCRIPTNAME%
REM :: SET ERRORLEVEL
SET ERRORLEVEL=0
REM :: SET PROXY-SERVER
set MyHTTPS_PROXY=http://xxx.xxx.xxx.xxx:3128
set MyHTTP_PROXY=http://xxx.xxx.xxx.xxx:3128
set TESTURL1=https://www.google.de/favicon.ico
set TESTURL2=https://www.debian.org/favicon.ico
REM :: SET REMOTE DOWNLOAD URL AND LOCAL FILENAME
set DOWNURL=https://hg.mozilla.org/releases/mozilla-release/raw-file/default/security/nss/lib/ckfw/builtins/certdata.txt
set DOWNFILE=certdata.txt
REM :: SET RELAX-MODE IN de-vs OR ONLY RELAX-STANDARD
REM :: Based on trustlist.txt of GnuGP VS Desktop, where "relax de-vs"
REM :: is used.
REM :: set RELAXEMODE=relax
set RELAXEMODE=relax de-vs
REM :: -------------------- config block ------------------------- END
REM :: 
REM :: This is a script for manual or automatic UPDATE of ROOT-Certs
REM :: from Mozilla's ROOT-CERTSTORE and automatic filtering and
REM :: conversion to GnuPG's trustlist-format in order to provide a
REM :: consistent validated cert-store for GnuPG.
REM :: 
REM :: This script may be launched under admin-rights by a weekly
REM :: windows-task. For security, the executables used by this script
REM :: have a modified Manifest-ressource that forces admin-rights on
REM :: execution in order to block normal execution by an
REM :: unrestrickted account.
REM :: 
REM :: -- Dependencies --
REM ::
REM :: - Admin-Rights
REM :: - Code-Locality of helper-tools in the same local path like
REM ::   this batch-script:
REM :: ==============================================================
REM ::   OpenSSL.exe      (X.509 fingerprinting, decoding)
REM ::   VBCertConv.exe   (Converter NSS to DER format)
REM ::   VBDownloader.exe (Downloader with hardened TLS-suites)
REM ::
REM :: - For proxy-support, see below under: "Download Database"
REM ::
REM :: ##############################################################
REM :: # !! Be aware !!
REM :: # Some certs with crt-endings are coded in base64 i. e. Dataport
REM :: # CA_05 and CA_05_01 (in original download format) AND NOT IN
REM :: # DER-format :-(
REM :: #
REM :: # Decode the data between "-----BEGIN CERTIFICATE-----" and
REM :: # "-----END CERTIFICATE-----" with a base64 -d infile outfile.crt
REM :: # -statement, before distributing ...
REM :: #
REM :: # For details, look here:
REM :: # https://www.ssl.com/guide/pem-der-crt-and-cer-x-509-encodings-and-conversions/
REM :: #
REM :: # The certificate "Dataport CA 05 (Original 2016)"
REM :: # http://pki.servicedpaor.de/ca/Dataport%20CA%2005.crt ...
REM :: # ... was deleted from Directory "extra-certs", because it has
REM :: # errors in its Certificate Policy (all urls are pointing to a
REM :: # non existend domain-name (http://pki.dataport.de). For
REM :: # Details, see PDF at: http://pki.servicedpaor.de/cps (P. 57).
REM :: ##############################################################
REM ::
REM ::   Download OpenSSL 32bit 1.1.1l from CURL-Projekt
REM ::   (now out-dated, we used a self-build version here)
REM :: ==============================================================
REM ::   https://curl.se/windows/
REM ::
REM :: ##############################################################
REM :: # Certs for Directory: "extra-certs" may be obtained from here:
REM :: ##############################################################
REM ::
REM ::   X.500-Bund certs
REM :: ==============================================================
REM ::   http://x500.bund.de/directory/f_nav_cert.html
REM ::
REM ::   Dataport certificates (ROOT-CA):
REM :: ==============================================================
REM ::   https://www.dataport.de/pki/zertifikate/
REM ::
REM ::   http://pki.servicedpaor.de/ca/Dataport%20Root%20CA%2001.crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20Root%20CA%2002.crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20Root%20CA%2002(1).crt
REM ::
REM ::   Dataport certificates (CA):
REM :: ==============================================================
REM ::   https://www.dataport.de/pki/zertifikate/
REM ::
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2001.crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2001(1).crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2002.crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2002(1).crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2003.crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2003(1).crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2003(2).crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2004.crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2004(1).crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2004(2).crt
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2005.crt     (NOT DER encoded ==> BASE64 !!)
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2005(1).crt  (NOT DER encoded ==> BASE64 !!)
REM ::   http://pki.servicedpaor.de/ca/Dataport%20CA%2006.crt
REM ::
REM ::   VerwaltungsPKI BUND
REM :: ==============================================================
REM ::   https://www.bsi.bund.de/DE/Themen/Oeffentliche-Verwaltung/Moderner-Staat/Verwaltungs-PKI/Wurzelzertifizierungsstelle/FingerprintsderWurzelzertifikate/fingerprintsderwurzelzertifikate_node.html
REM ::
REM ::   Bundeswehr
REM :: ==============================================================
REM ::   (!! Warning: ""Bw V-PKI CA 2019-1.der" is BASE64 encoded !!)
REM ::   # Decode the data between "-----BEGIN CERTIFICATE-----" and
REM ::   # "-----END CERTIFICATE-----" with a base64 -d infile outfile.crt
REM ::   # -statement, before distributing ...
REM ::   https://www.bundeswehr.de/de/organisation/ausruestung-baainbw/vergabe/public-key-infrastructure-der-bundeswehr-pkibw-
REM ::
REM ::   Telekom / Telesec
REM :: ==============================================================
REM ::   (!! Warning: "GlobalRoot_Class_2.crt" is BASE64 encoded !!)
REM ::   # Decode the data between "-----BEGIN CERTIFICATE-----" and
REM ::   # "-----END CERTIFICATE-----" with a base64 -d infile outfile.crt
REM ::   # -statement, before distributing ...
REM ::   https://corporate-pki.telekom.de/crt/GlobalRoot_Class_2.crt
REM ::   https://corporate-pki.telekom.de/crt/Deutsche_Telekom_AG_secure_email_CA_E03.crt
REM ::   https://corporate-pki.telekom.de/crt/Deutsche_Telekom_AG_secure_email_CA_E02.crt
REM ::   https://corporate-pki.telekom.de/crt/Deutsche_Telekom_AG_secure_email_CA.crt
REM ::   https://corporate-pki.telekom.de/crt/Deutsche_Telekom_AG_secure_email_CA_2.crt
REM ::   https://corporate-pki.telekom.de/crt/Deutsche_Telekom_AG_Issuing_CA_01.crt
REM ::
REM :: ##############################################################
REM :: # Information to directories from Gnupg.pdf:
REM :: ##############################################################
REM ::
REM :: Directory: "trusted-certs"
REM ::
REM :: This directory should be filled with certificates of Root CAs
REM :: you are trusting in checking the CRLs and signing OCSP
REM :: Responses. Usually these are the same certificates you use
REM :: with the applications making use of dirmngr. It is expected
REM :: that each of these certificate files contain exactly one DER
REM :: encoded certificate in a file with the suffix ‘.crt’ or
REM :: ‘.der’. dirmngr reads those certificates on startup and when
REM :: given a SIGHUP. Certificates which are not readable or do not
REM :: make up a proper X.509 certificate are ignored; see the log-
REM :: file for details.
REM ::
REM :: Applications using dirmngr (e.g. gpgsm) can request these
REM :: certificates to complete a trust chain in the same way as
REM :: with the extra-certs directory (see below). Note that for
REM :: OCSP responses the certificate specified using the option
REM :: ‘--ocsp-signer’ is always considered valid to sign OCSP
REM :: requests.
REM ::
REM :: Directory: "extra-certs"
REM ::
REM :: This directory may contain extra certificates which are
REM :: preloaded into the internal cache on startup. Applications
REM :: using dirmngr (e.g. gpgsm) can request cached certificates
REM :: to complete a trust chain. This is convenient in cases you
REM :: have a couple intermediate CA certificates or certificates
REM :: usually used to sign OCSP responses. These certificates are
REM :: first tried before going out to the net to look for them.
REM :: These certificates must also be DER encoded and suffixed with
REM :: ‘.crt’ or ‘.der’.
REM ::
REM :: File: "trustlist.txt"
REM :: Example:
REM :: https://dev.gnupg.org/source/gnupg/browse/master/doc/examples/trustlist.txt
REM :: ==============================================================
REM :: This is the list of trusted keys. You should backup this file.
REM :: Comment lines, indicated by a leading hash mark, as well as
REM :: empty lines are ignored. To mark a key as trusted you need
REM :: to enter its fingerprint followed by a space and a capital
REM :: letter S. Colons may optionally be used to separate the bytes
REM :: of a fingerprint; this enables cutting and pasting the
REM :: fingerprint from a key listing output. If the line is
REM :: prefixed with a ! the key is explicitly marked as not trusted.
REM :: Here is an example where two keys are marked as ultimately
REM :: trusted and the last one as not trusted:
REM ::
REM :: # CN=Wurzel ZS 3,O=Intevation GmbH,C=DE
REM :: A6935DD34EF3087973C706FC311AA2CCF733765B S
REM ::
REM :: # CN=PCA-1-Verwaltung-02/O=PKI-1-Verwaltung/C=DE
REM :: DC:BD:69:25:48:BD:BB:7E:31:6E:BB:80:D3:00:80:35:D4:F8:A6:CD S
REM ::
REM :: # CN=Root-CA/O=Schlapphuete/L=Pullach/C=DE
REM :: !14:56:98:D3:FE:9C:CA:5A:31:6E:BC:81:D3:11:4E:00:90:A3:44:C2 S
REM ::
REM :: Before entering a key into this file, you need to ensure its
REM :: authenticity. How to do this depends on your organisation;
REM :: your administrator might have already entered those keys
REM :: which are deemed trustworthy enough into this file. Places
REM :: where to look for the fingerprint of a root certificate are
REM :: letters received from the CA or the website of the CA (after
REM :: making 100% sure that this is indeed the website of that CA).
REM :: You may want to consider disallowing interactive updates of
REM :: this file by using the [option –no-allow-mark-trusted],
REM :: page 6. It might even be advisable to change the permissions
REM :: to read-only so that this file can’t be changed inadvertently.
REM :: As a special feature a line include-default will include a
REM :: global list of trusted certificates ...
REM :: ... (e.g. ‘/etc/gnupg/trustlist.txt’).
REM :: This global list is also used if the local list is not
REM :: available. It is possible to add further flags after the S
REM :: for use by the caller:
REM ::
REM :: relax
REM :: ==============================================================
REM ::       Relax checking of some root certificate requirements.
REM ::       As of now this flag allows the use of root certificates
REM ::       with a missing basic-Constraints attribute (despite
REM ::       that it is a MUST for CA certificates) and disables CRL
REM ::       checking for the root certificate.
REM :: cm
REM :: ==============================================================
REM ::       If validation of a certificate finally issued by a CA
REM ::       with this flag set fails, try again using the chain
REM ::       validation model.
REM ::
REM :: -- ChangeLog -- BEGIN
REM ::
REM :: 20210614
REM :: - Fixed removal of first chars in "CERT_SUBJECTX"
REM :: - Added consistency-check for "trustlist.txt"
REM ::
REM :: 20210903
REM :: - Added additional Informations and updated URLs in Comment.
REM ::
REM :: 20210929
REM :: - Added download function of Mozilla's cert-store
REM :: - Added conversion of Mozilla's cert-store into multiple DER-
REM ::   encoded certificate-files.
REM :: - Added errorlog
REM ::
REM :: 20220615
REM ::   (File: "trustlist.txt")
REM :: - Reverting back to colons for byte-separation in output of
REM ::   fingerprint, that is the "stdout" of openssl's output in
REM ::   order minimalize the errors in string-conversion by fu@#ing
REM ::   batch-script string-logic. The filtered fingerprint will
REM ::   also be written as a comment into the tustlist.txt.
REM :: - Adding flag "relax" to trust state in "trustlist.txt",
REM ::   because this is also used in the template of the version from
REM ::   "GnuPG VS Desktop".
REM ::
REM :: 20230105
REM :: - Enhanced cert-check after trustlist-build implemented, by
REM ::   reading the sha1-fingerprint from .der- and .crt-files.
REM ::   Errors are reported into the "trustlist_err.txt"-log.
REM :: - Cleanup "trusted-certs" before update.
REM :: - Now looking for ".crt" AND ".der" files for scanning and
REM ::   counting by using "findstr" 's capability in searching for
REM ::   multiple strings with line-count output of ":" as delimiter
REM ::   for "find"-command.
REM :: - Updated download-links of "extra-certs".
REM ::
REM :: 20230111
REM :: - "VBDownloader"-request modified, due to changed
REM ::   "VBDownloader" options after update.
REM ::
REM :: 20240617
REM :: - Updated to Version 2.3
REM :: - I missed adding the contents of directory "extra-certs" to
REM ::   "trustlist.txt" index-file, in order to make them visible to
REM ::   the "gpgsm"-process. Be aware, that gpgsm accepts Root CA
REM ::   certificates for its own purposes only if they are listed in
REM ::   its file "trustlist.txt". "dirmngr" does not make use of this
REM ::   list. Source:
REM ::   https://www.gnupg.org/documentation/manuals/gnupg/Dirmngr-Configuration.html
REM :: - Certificate-counting in error-checking extended to the
REM ::   content of directory "extra-certs".
REM ::
REM :: 20240619
REM :: - Updated to Version 2.4
REM :: - Changed download-path to "certdata.txt" in order to use the
REM ::   "release"-version of the "default"-revision, because i
REM ::   discovered an additional certificate:
REM ::   "FIRMAPROFESIONAL CA ROOT-A WEB" in the "tip"-revision of
REM ::   "certdata.txt". For details, see below under "DOWNLOAD-URL".
REM ::
REM :: 20240625
REM :: - Updated to Version 2.5
REM :: - Added dynamic proxy-support with predefined proxy.
REM ::
REM :: 20250123
REM :: - Added %RELAXEMODE% as RELAXEMODE=relax de-vs to
REM ::   trustlist-generation.
REM ::
REM :: -- ChangeLog -- END
REM ::
REM :: -- ToDo --
REM ::
REM :: - Use external config-file (.ini) for cert-URLs
REM ::   (no internal hard-coding)
REM :: - BASE64 => DER filter-coding ...
REM :: - Using absolute paths by %EXEDIR% for all operations due to
REM ::   security-considerations.
REM ::
REM :: ###############################################################
REM :: # Some coding hints ... BEGIN
REM :: ###############################################################
REM ::
REM :: # Example for ENABLEDELAYEDEXPANSION
REM :: ####################################
REM ::
REM :: set VAR=vorher
REM ::     if "%VAR%" == "vorher" (
REM ::         set VAR=nachher
REM ::         if "!VAR!" == "nachher" @echo Es funktioniert!
REM ::     )
REM ::     set LISTE=
REM ::     for %%i in (*) do set LISTE=!LISTE! %%i
REM ::     echo %LISTE%
REM :: 
REM :: # MessageBox by powershell Popup Method
REM :: #######################################
REM :: https://learn.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/windows-scripting/x83z1d9f(v=vs.84)
REM :: 
REM :: Chartables in VBA (0 - 127)
REM :: https://docs.microsoft.com/de-de/office/vba/language/reference/user-interface-help/character-set-0127
REM :: Chartables in VBA (128–255)
REM :: https://docs.microsoft.com/de-de/office/vba/language/reference/user-interface-help/character-set-128255
REM :: 
REM :: Syntax:
REM :: - intButton = object.Popup(strText,[nSecondsToWait],[strTitle],[nType])
REM :: 
REM :: Remarks:
REM :: - The Popup method displays a message box regardless of which
REM ::   host executable file is running (WScript.exe or CScript.exe).
REM :: - If "nSecondsToWait" equals zero (the default), the pop-up
REM ::   message box remains visible until closed by the user. If 
REM ::   "nSecondsToWaitis" is greater than zero, the pop-up message
REM ::   box closes after "nSecondsToWait" seconds.
REM :: - If you do not supply the argument "strTitle", the title of
REM ::   the pop-up message box defaults to "Windows Script Host."
REM :: 
REM :: Arguments:
REM :: - object
REM ::   WshShell object.
REM :: - strText
REM ::   String value that contains the text you want to appear in the
REM ::   pop-up message box.
REM :: - nSecondsToWait
REM ::   Optional. Numeric value indicating the maximum number of
REM ::   seconds you want the pop-up message box displayed. If
REM ::   "nSecondsToWait" is zero (the default), the pop-up message
REM ::   box remains visible until closed by the user.
REM :: - strTitle
REM ::   Optional. String value that contains the text you want to
REM ::   appear as the title of the pop-up message box.
REM :: - nType
REM ::   Optional. Numeric value indicating the type of buttons and
REM ::   icons you want in the pop-up message box. These determine how
REM ::   the message box is used.
REM :: - intButton
REM ::   Integer value indicating the number of the button the user
REM ::   clicked to dismiss the message box. This is the value
REM ::   returned by the Popup method.
REM :: 
REM :: BACKSPACE Chr(8)    / TAB     Chr(9)
REM :: LINEFEED  Chr(10)   / RETURN  Chr(13)
REM :: 
REM :: 196 Ä     Chr(196)  / 228 ä   Chr(228)
REM :: 213 Ö     Chr(213)  / 246 ö   Chr(246)
REM :: 220 Ü     Chr(220)  / 252 ü   Chr(252)
REM :: 223 ß     Chr(223)  / 128 €   Chr(128)
REM ::
REM :: MsgBox-constants
REM :: ====================================================
REM :: Button Types
REM :: Dec     Hex      Description
REM :: ====================================================
REM :: 0       0x0      Show OK button.
REM :: 1       0x1      Show OK and Cancel buttons.
REM :: 2       0x2      Show Abort, Retry, and Ignore buttons.
REM :: 3       0x3      Show Yes, No, and Cancel buttons.
REM :: 4       0x4      Show Yes and No buttons.
REM :: 5       0x5      Show Retry and Cancel buttons.
REM :: 6       0x6      Show Cancel, Try Again, and Continue buttons.
REM :: ====================================================
REM :: Icon Types
REM :: Dec     Hex      Description
REM :: ====================================================
REM :: 16      0x10     Show "Stop Mark" icon.
REM :: 32      0x20     Show "Question Mark" icon.
REM :: 48      0x30     Show "Exclamation Mark" icon.
REM :: 64      0x40     Show "Information Mark" icon.
REM :: ===============================================================
REM :: Other Type Values
REM :: Dec     Hex      Description
REM :: ===============================================================
REM :: 256     0x100    The second button is the default button.
REM :: 512     0x200    The third button is the default button.
REM :: 4096    0x1000   The message box is a system modal message box
REM ::                  and appears in a topmost window.
REM :: 524288  0x80000  The text is right-justified.
REM :: 1048576 0x100000 The message and caption text display in
REM ::                  right-to-left reading order, which is useful
REM ::                  for some languages.
REM :: ===============================================================
REM :: Return Value
REM :: The return value intButton is the number of the button that the
REM :: user clicked, or is -1 if the message box timed out. The
REM :: following table lists possible return values.
REM :: 
REM :: Dec     Description
REM :: ===============================================================
REM :: -1      The user did not click a button before "nSecondsToWait"
REM ::         seconds elapsed.
REM ::  1      OK button
REM ::  2      Cancel button
REM ::  3      Abort button
REM ::  4      Retry button
REM ::  5      Ignore button
REM ::  6      Yes button
REM ::  7      No button
REM ::  10     Try Again button
REM ::  11     Continue button
REM :: 
REM :: Example 1 MessageBox by wscript ComObject:
REM :: powershell.exe -ep Bypass -noprofile -command (new-object -ComObject wscript.shell).Popup('Systemh' + $([char]228) + 'rtung mittels Decrapifier beginnt ... ' + $([char]10) + $([char]13) + $([char]10) + $([char]13) + 'Diese Mitteilung wartet 10 Sekunden ...',10,'Information ...',0x0 + 0x40 + 0x1000) >nul 2>&1
REM :: Example 2 MessageBox by wscript ComObject with Return Value
REM :: (clicked button) to "stdout":
REM :: powershell.exe -ep Bypass -noprofile -command "(new-object -ComObject wscript.shell).Popup('Systemh' + $([char]228) + 'rtung mittels Decrapifier beginnt ... ' + $([char]10) + $([char]13) + $([char]10) + $([char]13) + 'Diese Mitteilung wartet 10 Sekunden ...',10,'Information ...',0x02 + 0x20 + 0x1000); exit $LASTEXITCODE"
REM :: 
REM :: ###############################################################
REM :: # Some coding hints ... END
REM :: ###############################################################

REM :: debug
REM :: echo on

REM :: SET DIMENSIONS
REM :: No scroll-buffer visible, when lines are defined
REM :: lower than vertical visual limit:
REM :: mode >nul 2>&1 && mode con cols=120 lines=40
REM :: mode >nul 2>&1 && mode con cols=120 lines=9999
color 0A

REM :: DOWNLOAD-URL
REM :: Using URLs from "urls"-array of curl's "mk-ca-bundle.pl"
REM :: https://github.com/curl/curl/blob/master/scripts/mk-ca-bundle.pl#L52
REM :: Due to the "tip"-revision state (see: https://serverfault.com/a/203346)
REM :: we will now use the "default" revision path of "certdata.txt":
REM :: ###############################################################
REM :: "tip"-revision path:
REM :: ###############################################################
REM :: https://hg.mozilla.org/mozilla-central/raw-file/tip/security/nss/lib/ckfw/builtins/certdata.txt
REM :: ###############################################################
REM :: "default"-revision path:
REM :: ###############################################################
REM :: nss:
REM :: https://hg.mozilla.org/projects/nss/raw-file/default/lib/ckfw/builtins/certdata.txt
REM :: central:
REM :: https://hg.mozilla.org/mozilla-central/raw-file/default/security/nss/lib/ckfw/builtins/certdata.txt
REM :: beta:
REM :: https://hg.mozilla.org/releases/mozilla-beta/raw-file/default/security/nss/lib/ckfw/builtins/certdata.txt
REM :: release:
REM :: https://hg.mozilla.org/releases/mozilla-release/raw-file/default/security/nss/lib/ckfw/builtins/certdata.txt
REM :: ###############################################################

REM :: info
goto cont0

:cont0
REM :: relative path to absolute path -DRIVE-
set drive=%~d0
set drivep=%drive%
If $#\#$==$#%drive:~-1%#$ set drivep=%drive:~0,-1%
set drivename=%drivep%

REM :: relative path to absolute path -PATH-
set pathn=%~p0
set pathp=%pathn%
If $#\#$==$#%pathn:~-1%#$ set pathp=%pathn:~0,-1%
set pathname=%pathp%

set HOMEDRIVE=%drivename%
set HOMEPATH=%pathname%
set EXEDIR=%HOMEDRIVE%%HOMEPATH%
set DIRCMD=/O:GNE

%HOMEDRIVE%
chdir %EXEDIR%

REM :: =====================================================================
REM :: Check admin-rights
REM :: =====================================================================
openfiles >nul 2>&1 || goto norights

REM :: Check GnuPG ROOT-CERT Directory
REM :: =====================================================================
if not exist "%EXEDIR%\trusted-certs" (
   set FERR="%EXEDIR%\trusted-certs"
   goto fileerr
)
if not exist "%EXEDIR%\extra-certs" (
   set FERR="%EXEDIR%\extra-certs"
   goto fileerr
)

REM :: Check OpenSSL        (Existence)/(Execution)
REM :: =====================================================================
set FERR="%EXEDIR%\OpenSSL.exe"
if not exist "%EXEDIR%\OpenSSL.exe" (
   goto fileerr
)
"%EXEDIR%\OpenSSL.exe" version >nul 2>&1 || goto fileerr

REM :: Check VBDownloader   (Existence)/(Execution)
REM :: =====================================================================
set FERR="%EXEDIR%\VBDownloader.exe"
if not exist "%EXEDIR%\VBDownloader.exe" (
   goto fileerr
)
"%EXEDIR%\VBDownloader.exe" -version >nul 2>&1 || goto fileerr

REM :: Check VBCertConv     (Existence)/(Execution)
REM :: =====================================================================
set FERR="%EXEDIR%\VBCertConv.exe"
if not exist "%EXEDIR%\VBCertConv.exe" (
   goto fileerr
)
"%EXEDIR%\VBCertConv.exe" -version >nul 2>&1 || goto fileerr

REM :: Check inet-connection
REM :: =====================================================================
set INETOK=TRUE

:inetbegin
REM :: Download Test
REM :: =====================================================================
color 09
echo.
echo Internetverbindung mit Download wird getestet.
echo Bitte warten ...
echo =====================================================================
echo.
"%EXEDIR%\VBDownloader.exe" -url "%TESTURL1%" -tofile "dummy1.ico" -wd "%EXEDIR%\trusted-certs" >nul 2>&1 || set INETOK=FALSE

REM :: Cleanup ...
del /f "%EXEDIR%\trusted-certs\*.ico" >nul 2>&1

if %INETOK%==TRUE (
    color 02
    echo Internet-Verbindung ohne Proxy ... OK
    echo Fahre fort ...
)

if %INETOK%==FALSE (
    color 04
    echo Internet-Verbindung ohne Proxy ... FEHLER
    echo Setze Proxy auf ...
    echo HTTPS_PROXY: %MyHTTPS_PROXY%
    echo HTTP_PROXY:  %MyHTTP_PROXY%
    set HTTPS_PROXY=%MyHTTPS_PROXY%
    set HTTP_PROXY=%MyHTTP_PROXY%
    set INETOK=TRUE
    "%EXEDIR%\VBDownloader.exe" -url "%TESTURL2%" -tofile "dummy2.ico" -wd "%EXEDIR%\trusted-certs" >nul 2>&1 || set INETOK=FALSE

    REM :: Cleanup ...
    del /f "%EXEDIR%\trusted-certs\*.ico" >nul 2>&1

    if %INETOK%==TRUE color 02
    if %INETOK%==TRUE echo Internet-Verbindung mit  Proxy ... OK
)
if %INETOK%==FALSE color 04
if %INETOK%==FALSE goto downloaderr

REM :: ### CLEANUP ###
REM :: CLEANUP "trusted-certs" when everything works fine.
REM :: =====================================================================
if exist "%EXEDIR%\trusted-certs" (
    echo.
    echo.Bereinige Zertifikatsverzeichnis: "trusted-certs"
    del /f "%EXEDIR%\trusted-certs\*.crt" >nul 2>&1
    del /f "%EXEDIR%\trusted-certs\*.der" >nul 2>&1
)

REM :: Download Database
REM :: =====================================================================
color 09
echo.
echo Die Zertifikatsdatenbank wird heruntergeladen.
echo Bitte warten ...
echo =====================================================================
echo.
REM :: "%EXEDIR%\VBDownloader.exe" -url "%DOWNURL%" -tofile "%DOWNFILE%" -wd "%EXEDIR%\trusted-certs" -proxy "xxx.xxx.xxx.xxx:3128" || goto downloaderr
"%EXEDIR%\VBDownloader.exe" -url "%DOWNURL%" -tofile "%DOWNFILE%" -wd "%EXEDIR%\trusted-certs" || goto downloaderr

REM :: Transcode Database
REM :: =====================================================================
color 0B
echo.
echo Die Zertifikatsdatenbank wird transkodiert.
echo Bitte warten ...
echo =====================================================================
"%EXEDIR%\VBCertConv.exe" -wd "%EXEDIR%\trusted-certs" -to-files || goto codererr

REM :: Build Trustlist
REM :: =====================================================================
:cont1
color 0F
echo.
echo Die Trustliste wird aufgebaut.
echo Bitte warten ...
echo =====================================================================
echo.
REM :: Initialer Aufbau der ROOT-CERT Trusliste
echo.################################################################################################################>  "trustlist.txt"
echo.#                                   List of Root CAs fully trusted by gpgsm                                    #>> "trustlist.txt"
echo.#            Data-Source are the TRUSTED entries of the database from Mozilla's ROOT-Cert store ...            #>> "trustlist.txt"
echo.#        https://git.gnupg.org/cgi-bin/gitweb.cgi?p=gpg4win.git;a=blob;f=doc/HOWTO-SMIME.de.txt;hb=HEAD        #>> "trustlist.txt"
echo.# %DOWNURL% #>> "trustlist.txt"
echo.################################################################################################################>> "trustlist.txt"
echo.>> "trustlist.txt"
REM :: echo ###########################################  > "trustlist.txt"
REM :: echo # GnuPG X.509 ROOT-CERT-Store Trust-Liste # >> "trustlist.txt"
REM :: echo ########################################### >> "trustlist.txt"

REM :: Datum filtern
REM :: FOR /F " usebackq tokens=1,2* delims= " %%a IN (`date /t`) DO (
REM ::     echo # Datum: %%a %%b >> "trustlist.txt"
REM :: )

REM :: Uhrzeit filtern
REM :: FOR /F " usebackq tokens=1,2* delims= " %%a IN (`time /t`) DO (
REM ::     echo # Uhrzeit: %%a >> "trustlist.txt"
REM :: )
REM :: echo ########################################### >> "trustlist.txt"

REM :: Initialer Aufbau der ROOT-CERT Trusliste-Fehlerliste
echo ###########################################  > "trustlist_err.txt"
echo # GnuPG X.509 ROOT-CERT-Store Trust-Liste # >> "trustlist_err.txt"
echo ########################################### >> "trustlist_err.txt"

REM :: Datum filtern
FOR /F " usebackq tokens=1,2* delims= " %%a IN (`date /t`) DO (
    echo # Datum: %%a %%b >> "trustlist_err.txt"
)

REM :: Uhrzeit filtern
FOR /F " usebackq tokens=1,2* delims= " %%a IN (`time /t`) DO (
    echo # Uhrzeit: %%a >> "trustlist_err.txt"
)
echo ########################################### >> "trustlist_err.txt"

REM :: GET FINGERPRINT FROM CERT
REM :: openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in file.crt 2>nul
REM :: OUTPUT MUST NOT BE FILTERED, DUE TO ":" STRING IN OUTPUT,
REM :: because GnuPG is able to read both formats.
REM :: Example:
REM :: https://dev.gnupg.org/source/gnupg/browse/master/doc/examples/trustlist.txt
REM ::
REM :: Example -fingerprint: SHA1 Fingerprint=FF:BD:CD:E7:82:C8:43:5E:3C:6F:26:86:5C:CA:A8:3A:45:5B:C3:0A
REM :: Example -subject: subject= /C=DE/O=PKI-1-Verwaltung/CN=PCA-1-Verwaltung-11
REM :: Remove ":"-sign from STRING
REM :: SET no_doublepoint=%hash::=%

REM :: Filter Subject und Fingerprint aus Zertifikat und indiziere in
REM :: die Trust-Liste aus Dir "trusted-certs".
REM ::
FOR /F " usebackq tokens=1,2* delims=" %%i IN (`dir "trusted-certs\*.crt" "trusted-certs\*.der" /b 2^>nul`) DO (
    SET CERTFILE=%%i
    echo Bearbeite: %%i ...
    REM :: Decode Subject for indexing ...
    FOR /F " usebackq tokens=1,2* delims=" %%s IN (`openssl.exe x509 -noout -subject -inform DER -in "trusted-certs\!CERTFILE!" 2^>^>trustlist_err.txt`) DO (
        SET CERT_SUBJECT=%%s
        
        REM :: Remove first 8 chars from openssl output ("subject= ")
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        SET "CERT_SUBJECTX=!CERT_SUBJECT:~8!"
        echo # !CERT_SUBJECTX!>> "trustlist.txt"
        
        REM :: Remove Spaces from from openssl output ("subject= ")
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        REM :: SET "CERT_SUBJECTY=!CERT_SUBJECTX: =!"
        REM :: echo # !CERT_SUBJECTY!>> "trustlist.txt"
    )
    REM :: Decode fingerprint and convert for indexing ...
    FOR /F " usebackq tokens=1,2* delims==" %%l IN (`openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in "trusted-certs\!CERTFILE!" 2^>^>trustlist_err.txt`) DO (
        SET HASH_DOUBLE=%%m
        REM :: Remove all ":" chars from openssl output to convert fingerprint-format
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        REM :: echo # OpenSSL Fingerprint: !HASH_DOUBLE! >> "%ProgramData%\GNU\etc\gnupg\trustlist.txt"
        REM :: SET "HASH_NO_DOUBLE=!HASH_DOUBLE::=!"
        REM :: echo # !HASH_NO_DOUBLE! S %RELAXEMODE%>> "trustlist.txt"
        echo !HASH_DOUBLE! S %RELAXEMODE%>> "trustlist.txt"
        REM :: !! WICHTIG (siehe Dokumentation) !!
        REM :: Schreibe eine Leerzeile nach jedem Eintrag und in letze Zeile ...
        echo.>> "trustlist.txt"
    )
)

REM :: Filter Subject und Fingerprint aus Zertifikat und indiziere in
REM :: die Trust-Liste aus Dir "extra-certs".
REM ::
FOR /F " usebackq tokens=1,2* delims=" %%i IN (`dir "extra-certs\*.crt" "extra-certs\*.der" /b 2^>nul`) DO (
    SET CERTFILE=%%i
    echo Bearbeite: %%i ...
    REM :: Decode Subject for indexing ...
    FOR /F " usebackq tokens=1,2* delims=" %%s IN (`openssl.exe x509 -noout -subject -inform DER -in "extra-certs\!CERTFILE!" 2^>^>trustlist_err.txt`) DO (
        SET CERT_SUBJECT=%%s
        
        REM :: Remove first 8 chars from openssl output ("subject= ")
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        SET "CERT_SUBJECTX=!CERT_SUBJECT:~8!"
        echo # !CERT_SUBJECTX!>> "trustlist.txt"
        
        REM :: Remove Spaces from from openssl output ("subject= ")
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        REM :: SET "CERT_SUBJECTY=!CERT_SUBJECTX: =!"
        REM :: echo # !CERT_SUBJECTY!>> "trustlist.txt"
    )
    REM :: Decode fingerprint and convert for indexing ...
    FOR /F " usebackq tokens=1,2* delims==" %%l IN (`openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in "extra-certs\!CERTFILE!" 2^>^>trustlist_err.txt`) DO (
        SET HASH_DOUBLE=%%m
        REM :: Remove all ":" chars from openssl output to convert fingerprint-format
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        REM :: echo # OpenSSL Fingerprint: !HASH_DOUBLE! >> "%ProgramData%\GNU\etc\gnupg\trustlist.txt"
        REM :: SET "HASH_NO_DOUBLE=!HASH_DOUBLE::=!"
        REM :: echo # !HASH_NO_DOUBLE! S %RELAXEMODE%>> "trustlist.txt"
        echo !HASH_DOUBLE! S %RELAXEMODE%>> "trustlist.txt"
        REM :: !! WICHTIG (siehe Dokumentation) !!
        REM :: Schreibe eine Leerzeile nach jedem Eintrag und in letze Zeile ...
        echo.>> "trustlist.txt"
    )
)

REM :: Check trustlist.txt consistency
REM :: Count all cert-files by counting the occurence of ":" in output
REM :: of "findstr" by "find" over the directories:
REM :: "trusted-certs" and "extra-certs".
for /f %%a in ('dir "trusted-certs\*.crt" "trusted-certs\*.der" "extra-certs\*.crt" "extra-certs\*.der" /b 2^>nul ^|findstr /I /N ".crt .der" ^|find ":" /c') do set /a EXISTED_FILES=%%a
for /f %%b in ('type "trustlist.txt"^|find " S " /c') do set /a SCANNED_CERTS=%%b

REM ::
REM :: Check cert consistency by reading fingerprint
REM ::
SET CERTERROR=0
echo.
echo =====================================================================
echo.Teste Zertifikate im Verzeichnis "trusted-certs" ...
echo =====================================================================
set osslchk001err=0
FOR /F " usebackq delims==" %%i IN (`dir /b "trusted-certs\*.crt" "trusted-certs\*.der" 2^>nul`) DO (
    openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in "trusted-certs\%%i" >nul 2>&1 || set osslchk001err=1
    if "!osslchk001err!" == "1" (
        echo.Fehler in Zertifikats-Datei: "trusted-certs\%%i"
        echo.                                                 >>trustlist_err.txt
        echo.Warnung ...                                      >>trustlist_err.txt
        echo.Fehler in Zertifikats-Datei: "trusted-certs\%%i" >>trustlist_err.txt
        set osslchk001err=0
        SET CERTERROR=1
    )
)
if "%CERTERROR%" == "0" (
    echo.Keine Fehler ...
)

echo.
echo =====================================================================
echo.Teste Zertifikate im Verzeichnis "extra-certs" ...
echo =====================================================================
set osslchk002err=0
FOR /F " usebackq delims==" %%i IN (`dir /b "extra-certs\*.crt" "extra-certs\*.der" 2^>nul`) DO (
    openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in "extra-certs\%%i" >nul 2>&1 || set osslchk002err=1
    if "!osslchk002err!" == "1" (
        echo.Fehler in Zertifikats-Datei: "extra-certs\%%i"
        echo.                                                 >>trustlist_err.txt
        echo.Warnung ...                                      >>trustlist_err.txt
        echo.Fehler in Zertifikats-Datei: "trusted-certs\%%i" >>trustlist_err.txt
        set osslchk002err=0
        SET CERTERROR=1
    )
)
if "%CERTERROR%" == "0" (
    echo.Keine Fehler ...
)
if "%EXISTED_FILES%" == "" goto trustlist_error
if "%SCANNED_CERTS%" == "" goto trustlist_error
if "%CERTERROR%" == "1" goto trustlist_error
echo.
echo =====================================================================
echo Verarbeitete Zertifikatsinhalte: %SCANNED_CERTS%
echo Verarbeitete Zertifikatsdateien: %EXISTED_FILES%
echo =====================================================================
if "%EXISTED_FILES%" == "%SCANNED_CERTS%" goto end
goto trustlist_error

:tmperror
SET ERRORLEVEL=1
color 04
echo.
echo (%SCRIPTNAME%) FEHLER:
echo Die Variable fuer das Temp-Verzeichnis existiert nicht,
echo oder keine Schreibrechte in "TEMP" oder "TMP".
echo.
goto end

:fileerr
SET ERRORLEVEL=1
color 04
echo.
echo (%SCRIPTNAME%) FEHLER:
echo %FERR% ...
echo ... konnte nicht gefunden oder gestartet werden.
echo Bitte pruefen ...
echo.
goto end

:norights
SET ERRORLEVEL=1
color 04
echo.
echo (%SCRIPTNAME%) FEHLER:
echo Sie besitzen KEINE administrativen Rechte.
echo.
goto end

:downloaderr
SET ERRORLEVEL=1
color 04
echo.
echo (%SCRIPTNAME%) FEHLER:
echo Fehler beim Download ...
echo.
goto end

:codererr
SET ERRORLEVEL=1
color 04
echo.
echo (%SCRIPTNAME%) FEHLER:
echo Fehler beim Transkodieren ...
echo.
goto end

:trustlist_error
SET ERRORLEVEL=1
color 04
echo.
echo (%SCRIPTNAME%) WARNUNG:
echo Eine Inkonsistenz ist bei der Verarbeitung der Zertifikate
echo aufgetreten. Die Anzahl der verarbeiteten Zertifikate
echo entspricht NICHT der Anzahl der Zertifikatsdateien.
echo Schauen Sie in die Datei: "trustlist_err.txt" ...
echo.
goto end

:end
if exist "%EXEDIR%\trusted-certs\certdata.txt" (
    echo.
    echo Cleanup: "trusted-certs\certdata.txt"
    del /f "%EXEDIR%\trusted-certs\certdata.txt" >nul 2>&1
)
echo ERRORLEVEL: %ERRORLEVEL%
echo (%SCRIPTNAME%) beendet.
REM :: echo Bitte Taste ...
REM :: pause >nul
color
EXIT /B %ERRORLEVEL%
ENDLOCAL
