@echo off & SETLOCAL & SETLOCAL ENABLEDELAYEDEXPANSION & CLS & echo.

REM :: Version 2.2 by Veit Berwig in 01/2023
REM ::
REM :: INFO:
REM :: Script zum Anzeigen von Teilinformationen der Zertifkatsdateien.
REM :: Hiermit kann man sehen, ob "openssl" die Dateien korrekt lesen
REM :: kann, da manchmal UNICODE Sonderzeichen von Zertifikaten aus
REM :: anderen Lndern in den Dateinamen zu Lesefehlern fhren.
REM :: 
REM :: Changelog 
REM :: 
REM :: 20230106
REM :: 
REM :: - Redesign with error-checking above all relevant directories
REM ::   and providing output of "EXITSTATUS:0" / "EXITSTATUS:1"
REM ::   in STDOUT.
REM ::
REM ::   To test errorhandling (see above under "INFO:"), download ...
REM ::
REM ::   https://corporate-pki.telekom.de/crt/GlobalRoot_Class_2.crt
REM ::
REM ::   ... and save it under dir "trusted-certs" or "extra-certs"
REM ::   with a name like "GlobalRoot_Class_2_defect.crt". This cert
REM ::   is wrong encoded in "base64" and not in "der"-format, so
REM ::   "openssl" will throw an error here !!
REM :: 

:cont0
REM :: relative path to absolute path -DRIVE-
set drive=%~d0
set drivep=%drive%
If $#\#$==$#%drive:~-1%#$ set drivep=%drive:~0,-1%
set drivename=%drivep%

REM :: relative path to absolute path -PATH-
set pathn=%~p0
set pathp=%pathn%
If $#\#$==$#%pathn:~-1%#$ set pathp=%pathn:~0,-1%
set pathname=%pathp%

set HOMEDRIVE=%drivename%
set HOMEPATH=%pathname%
set EXEDIR=%HOMEDRIVE%%HOMEPATH%
set DIRCMD=/O:GNE

REM :: Set ERROR_TOKEN
set CERTERROR=0

%HOMEDRIVE%
chdir %EXEDIR%

REM :: =====================================================================
REM :: Check admin-rights
REM :: =====================================================================
openfiles >nul 2>&1 || goto norights

echo.
echo.Bearbeite: Verzeichnis "trusted-certs" ...
echo.=================================================================
FOR /F " usebackq tokens=1,2* delims=" %%i IN (`dir "trusted-certs\*.crt" "trusted-certs\*.der" /b 2^>nul`) DO (
    SET CERTFILE=%%i
    echo.Bearbeite: "trusted-certs\%%i":
    echo.=================================================================
    REM :: Decode Subject for indexing ...
    SET CERT_SUBJECT=0
    FOR /F " usebackq tokens=1,2* delims=" %%s IN (`openssl.exe x509 -noout -subject -inform DER -in "trusted-certs\!CERTFILE!" 2^>nul`) DO (
        SET CERT_SUBJECT=%%s
        REM :: Remove first 8 chars from openssl output ("subject= ")
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        SET "CERT_SUBJECTX=!CERT_SUBJECT:~8!"
        REM :: Remove Spaces from from openssl output ("subject= ")
        REM :: SET "CERT_SUBJECTY=!CERT_SUBJECTX: =!"
        REM :: echo # !CERT_SUBJECTY!
        echo.# !CERT_SUBJECTX!
    )
    REM :: We cannot use an || error-pipe in FOR-IN-execution-statement
    REM :: above between (`...`), so we have to do an error-handling
    REM :: by pre-defined environment-var "CERT_SUBJECT"=0 ...
    if "!CERT_SUBJECT!" == "0" (
        echo.
        echo.#################################################################
        echo.# Warnung ...
        echo.#################################################################
        echo.Fehler in Zertifikats-Datei: "trusted-certs\%%i"
        echo.#################################################################
        echo.
        set CERTERROR=1
    )
    REM :: Decode fingerprint and convert for indexing ...
    FOR /F " usebackq tokens=1,2* delims==" %%l IN (`openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in "trusted-certs\!CERTFILE!" 2^>nul`) DO (
        SET HASH_DOUBLE=%%m
        REM :: Remove all ":" chars from openssl output to convert fingerprint-format
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        REM :: echo # OpenSSL Fingerprint: !HASH_DOUBLE! >> "%ProgramData%\GNU\etc\gnupg\trustlist.txt"
        REM :: SET "HASH_NO_DOUBLE=!HASH_DOUBLE::=!"
        REM :: echo # !HASH_NO_DOUBLE! S relax>> "trustlist.txt"
        echo !HASH_DOUBLE! S relax
        REM :: !! WICHTIG (siehe Dokumentation) !!
        REM :: Schreibe eine Leerzeile nach jedem Eintrag und in letze Zeile ...
        echo.
    )
)

echo.Bearbeite: Verzeichnis "extra-certs" ...
echo.=================================================================
FOR /F " usebackq tokens=1,2* delims=" %%i IN (`dir "extra-certs\*.crt" "extra-certs\*.der" /b 2^>nul`) DO (
    SET CERTFILE=%%i
    echo.Bearbeite: "extra-certs\%%i":
    echo.=================================================================
    REM :: Decode Subject for indexing ...
    SET CERT_SUBJECT=0
    FOR /F " usebackq tokens=1,2* delims=" %%s IN (`openssl.exe x509 -noout -subject -inform DER -in "extra-certs\!CERTFILE!" 2^>nul`) DO (
        SET CERT_SUBJECT=%%s
        REM :: Remove first 8 chars from openssl output ("subject= ")
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        SET "CERT_SUBJECTX=!CERT_SUBJECT:~8!"
        REM :: Remove Spaces from from openssl output ("subject= ")
        REM :: SET "CERT_SUBJECTY=!CERT_SUBJECTX: =!"
        REM :: echo # !CERT_SUBJECTY!
        echo.# !CERT_SUBJECTX!
    )
    REM :: We cannot use an || error-pipe in FOR-IN-execution-statement
    REM :: above between (`...`), so we have to do an error-handling
    REM :: by pre-defined environment-var "CERT_SUBJECT"=0 ...
    if "!CERT_SUBJECT!" == "0" (
        echo.
        echo.#################################################################
        echo.# Warnung ...
        echo.#################################################################
        echo.Fehler in Zertifikats-Datei: "extra-certs\%%i"
        echo.#################################################################
        echo.
        set CERTERROR=1
    )
    REM :: Decode fingerprint and convert for indexing ...
    FOR /F " usebackq tokens=1,2* delims==" %%l IN (`openssl.exe x509 -noout -fingerprint -sha1 -inform DER -in "extra-certs\!CERTFILE!" 2^>nul`) DO (
        SET HASH_DOUBLE=%%m
        REM :: Remove all ":" chars from openssl output to convert fingerprint-format
        REM :: https://www.dostips.com/DtTipsStringManipulation.php
        REM :: echo # OpenSSL Fingerprint: !HASH_DOUBLE! >> "%ProgramData%\GNU\etc\gnupg\trustlist.txt"
        REM :: SET "HASH_NO_DOUBLE=!HASH_DOUBLE::=!"
        REM :: echo # !HASH_NO_DOUBLE! S relax>> "trustlist.txt"
        echo !HASH_DOUBLE! S relax
        REM :: !! WICHTIG (siehe Dokumentation) !!
        REM :: Schreibe eine Leerzeile nach jedem Eintrag und in letze Zeile ...
        echo.
    )
)

goto end

:norights
echo.
echo FEHLER:
echo Sie besitzen KEINE administrativen Rechte.
echo.
goto end

:end
echo.EXITSTATUS:%CERTERROR%
REM :: echo Bitte Taste ...
REM :: pause >nul
EXIT /B %CERTERROR%
ENDLOCAL
